import pykf
from Globals import InitializeClass, Persistent, PersistentMapping
from Acquisition import Implicit, aq_parent, aq_base, aq_inner, aq_chain, aq_self
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from OFS.Traversable import Traversable
from Products.CMFCore.CMFCorePermissions import View, AccessContentsInformation
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.Link import Link



factory_type_information = (
  { 'id'             : 'Trackback Ping'
  , 'meta_type'      : 'Trackback Ping'
  , 'description'    : 'Movabletype Trackback'
  , 'icon'           : 'link_icon.gif'
  , 'product'        : '' # leave blank to suppress
  , 'factory'        : ''
  , 'immediate_view' : ''
  , 'actions'        : ( { 'id'            : 'view'
                         , 'name'          : 'View'
                         , 'action': 'string:${object_url}/link_view'
                         , 'permissions'   : (View,)
                         }
                       ,
                       )
  }
,
)


def charconv(txt, site_charset='euc-jp'):
    site_charset = site_charset.lower()
    c = pykf.guess(txt)

    if c is pykf.UTF8:
        try:
            return unicode(txt, 'utf8').encode(site_charset, 'replace')
        except LookupError:
            raise LookupError, 'can not convert utf-8 to %s' % site_charset

    if site_charset == 'euc-jp':
        return pykf.toeuc(txt, c)
    elif site_charset == 'iso-2022-jp':
        return pykf.tojis(txt, c)
    elif site_charset == 'shift_jis':
        return pykf.tosjis(txt, c)
    else:
        return txt


class Ping(Link):
    """
    Trackback ping
    """

    meta_type = 'Trackback Ping'
    portal_type = 'Trackback Ping'
    review_state = 'published'
    __implements__ = ( Link.__implements__ , )

                                              
    security = ClassSecurityInfo()

    def __init__(self, id, title, excerpt, url, blog_name):
        self.blog_name = blog_name
        Link.__init__(self, id, title, url, excerpt)

    security.declarePublic('Creator')
    def Creator(self):
        """
        Return blog_name
        """
        return self.blog_name

InitializeClass(Ping)


class PingContainer(Persistent, Implicit, Traversable):
    """
    Store trackback ping
    """

    id = 'trackback'

    security = ClassSecurityInfo()

    index_html = None

    def __init__(self):
        self.id = 'trackback'
        self._container = PersistentMapping()

    security.declareProtected(View, 'getId')
    def getId(self):
        return self.id

    security.declareProtected(View, 'getPing')
    def getPing(self, id):
        """
        return ping
        """
        return self._container.get(id).__of__(self)

    security.declareProtected(View, 'addPing')
    def addPing(self, title, excerpt, url, blog_name, site_charset):
        """
        add trackback ping
        """
        container = self._container

        id = int(DateTime().timeTime())
        while self._container.get(str(id), None) is not None:
            id = id + 1
        id = str(id)

        title = charconv(title, site_charset)
        excerpt = charconv(excerpt, site_charset)
        blog_name = charconv(blog_name, site_charset)

        item = Ping(id, title, excerpt, url, blog_name)
        item.__of__(self).indexObject()
        self._container[id] = item

    security.declareProtected(View, '__bobo_traverse__')
    def __bobo_traverse__(self, REQUEST, name):
        """
        traversal
        """
        target = getattr(self, name, None)
        if target:
            return target
        else:
            try:
                return self.getPing(name)
            except:
                p = self.aq_inner.aq_parent
                if p.getId() == name:
                    return p
                else:
                    REQUEST.RESPONSE.notFoundError("%s\n%s" % (name, ''))

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """
        """
        if aq_base(container) is not aq_base(self):
            for obj in self.objectValues():
                obj.__of__(self).manage_afterAdd(item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        """
        """
        for obj in self.objectValues():
            obj.__of__(self).manage_afterClone(item)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        if aq_base(container) is not aq_base(self):
            for obj in self.objectValues():
                obj.__of__(self).manage_beforeDelete(item, container)

    security.declareProtected(AccessContentsInformation, 'objectIds')
    def objectIds( self, spec=None ):
        """
        """
        if spec and spec is not Ping.meta_type:
            return []
        return self._container.keys()

    security.declareProtected(AccessContentsInformation, 'objectItems')
    def objectItems(self, spec=None):
        """
        """
        r=[]
        a=r.append
        g=self.getPing
        for id in self.objectIds(spec):
            a((id, g(id)))
        return r

    security.declareProtected(AccessContentsInformation, 'objectValues')
    def objectValues(self, spec=None):
        """
        """
        return map(self.getPing, self.objectIds(spec))
            

InitializeClass(PingContainer)
